/*
Prosty program, który miga kierunkowskazami oraz okresowo włącza i wyłącza światła stop oraz światła drogowe.
Kierunkowskazy mogą działać niezależnie od siebie, ale mogą też działać wspólnie jako światła awaryjne.
Czasy działania kierunkowskazów, świateł stop oraz świateł drogowych są bazowane na stałych wartościach.
Czas migania oraz świecenia oparty jest na "cyklach migania", które są używane do liczników świecenia oraz przerw.
*/
 
 // Stałe techniczne odpowiednie do platformy Arduino
const byte PIN_KierunkowskazyLewy = 8; // Numer PIN dla lewego kierunkowskazu
const byte PIN_KierunkowskazyPrawy = 7; // Numer PIN dla prawego kierunkowskazu
const byte PIN_SwiatlaStop = 6; // Numer PIN dla świateł stop
const byte PIN_SwiatlaDrogowe = 5; // Numer PIN dla świateł drogowych

// Stałe definiujące działanie całego programu
const int Delay_Kierunkowskazy = 400; // Czas jaki kierunkowskaz ma świecić oraz nie swiecić (ms), to jest czas dla funkcji delay()
const byte Cykle_Kierunkowskazy = 7; // Domyślna ilość cykli do wykonania dla kierunkowskazu lewego, prawego lub awaryjnych
const byte Cykle_KierunkowskazyPrzerwa = 5; // Domyślna ilość cykli przerwy pomiędzy kierunkowskazami
const byte Cykle_SwiatlaStop = 5; // Domyślna ilość cykli do wykonania dla świateł stop
const byte Cykle_SwiatlaStopPrzerwa = 7; // Domyślna ilość cykli przerwy dla świateł stop
const byte Cykle_SwiatlaDrogowe = 7; // Domyślna ilość cykli do wykonania dla świateł drogowych
const byte Cykle_SwiatlaDrogowePrzerwa = 9; // Domyślna ilość cykli przerwy dla świateł drogowych

// Zmienne definiujące działanie całego programu
byte Licznik_KierunkowskazyLewy = 0; // Ilość cykli do wykonania dla kierunkowskazu lewego
byte Licznik_KierunkowskazyPrawy = 0; // Ilość cykli do wykonania dla kierunkowskazu prawego
byte Licznik_KierunkowskazyPrzerwa = 0; // Ilość cykli przerwy pomiędzy kierunkowskazami
byte Licznik_SwiatlaStop = 0; // Ilość cykli do wykonania dla swiateł stop
byte Licznik_SwiatlaStopPrzerwa = 0; // Ilość cykli przerwy dla świateł stop
byte Licznik_SwiatlaDrogowe = 0; // Ilość cykli do wykonania dla swiateł drogowych
byte Licznik_SwiatlaDrogowePrzerwa = 0; // Ilość cykli przerwy dla świateł drogowych

// Początkowa konfiguracja platformy Adruino oraz zmiennych całego programu
void setup() {
  
  // Inicjalizacja PIN'ów platformy Adruino
  pinMode(PIN_KierunkowskazyLewy, OUTPUT); // Inicjalizacja PIN dla lewego kierunkowskazu
  pinMode(PIN_KierunkowskazyPrawy, OUTPUT); // Inicjalizacja PIN dla prawego kierunkowskazu
  pinMode(PIN_SwiatlaStop, OUTPUT); // Inicjalizacja PIN dla świateł stop
  pinMode(PIN_SwiatlaDrogowe, OUTPUT); // Inicjalizacja PIN dla świateł drogowych

  // Inicjalizacja generatora losowego poprzez nie przypięty PIN #0
  randomSeed(analogRead(A0));

  // Startujemy z kierunkowskazami awaryjnymi
  Licznik_KierunkowskazyLewy = Cykle_Kierunkowskazy;
  Licznik_KierunkowskazyPrawy = Licznik_KierunkowskazyLewy;
  
  // Startujemy z działającymi światłami stop
  Licznik_SwiatlaStop = Cykle_SwiatlaStop;
  digitalWrite(PIN_SwiatlaStop, HIGH); // Włączanie świateł stop

  // Startujemy z działającymi światłami drogowymi
  Licznik_SwiatlaDrogowe = Cykle_SwiatlaDrogowe;
  digitalWrite(PIN_SwiatlaDrogowe, HIGH); // Włączanie świateł drogowych
  
}

// Główna pętla działania platformy Adruino oraz całego programu
void loop() {
  
  // Program opiera się na liczeniu cyklów kierunkowskazów, więc zawsze robimy jeden cykl migania nawet jak kierunkowskazy nie mają działać
  
  // Operacje do wykonania jeśli ma działać kierunkowskaz lewy
  if ( Licznik_KierunkowskazyLewy > 0 ) {
    digitalWrite(PIN_KierunkowskazyLewy, HIGH); // Włączanie lewego kierunkowskazu
    // Zmniejszamy ilość cykli, które mają być jeszcze wykonane
    Licznik_KierunkowskazyLewy = Licznik_KierunkowskazyLewy - 1;
  }
  
  // Operacje do wykonania jeśli ma działać kierunkowskaz prawy
  if ( Licznik_KierunkowskazyPrawy > 0) {
    digitalWrite(PIN_KierunkowskazyPrawy, HIGH); // Włączanie prawego kierunkowskazu
    // Zmniejszamy ilość cykli, które mają być jeszcze wykonane
    Licznik_KierunkowskazyPrawy = Licznik_KierunkowskazyPrawy - 1;
  }
  
  // Odczekanie minimalnego czasu świecenia kierunkowskazów
  delay(Delay_Kierunkowskazy);
  
  // Wyłączenie obu kierunkowskazów nie dbając o to, czy były wcześniej włączone
  digitalWrite(PIN_KierunkowskazyLewy, LOW); // Wyłączanie lewego kierunkowskazu
  digitalWrite(PIN_KierunkowskazyPrawy, LOW); // Wyłączanie prawego kierunkowskazu
  
  // Odczekanie minimalnego czasu wyłączenia kierunkowskazów
  delay(Delay_Kierunkowskazy);
  
  // Skończył się cykl migania, więc trzeba zdecydować co należy zrobić dalej
  
  // Operacje do wykonania jeśli aktualnie jest przerwa w działaniu kierunkowskazów
  if ( Licznik_KierunkowskazyPrzerwa > 0 ) {
    // Zmniejszamy ilość cykli, które mają być jeszcze wykonane
    Licznik_KierunkowskazyPrzerwa = Licznik_KierunkowskazyPrzerwa - 1;
    // Jeśli przerwa się skończyła to losujemy jakie kierunkowskazy będą następnie działać
    if ( Licznik_KierunkowskazyPrzerwa == 0 ) {
      switch (random(0, 3)) {
        case 0: // Będzie działał kierunkowskaz lewy, więc ustawiamy dla niego licznik cyklów
          Licznik_KierunkowskazyLewy = Cykle_Kierunkowskazy;
          break;
        case 1: // Będzie działał kierunkowskaz prawy, więc ustawiamy dla niego licznik cyklów
          Licznik_KierunkowskazyPrawy = Cykle_Kierunkowskazy;
          break;
        default: // Będą działały kierunkowskazy awaryjne, więc ustawiamy dla obu ten sam licznik cyklów
          Licznik_KierunkowskazyLewy = Cykle_Kierunkowskazy;
          Licznik_KierunkowskazyPrawy = Licznik_KierunkowskazyLewy;
          break;
      }
    }
  }
  // Operacje do wykonania jeśli nie było przerwy w działaniu kierunkowskazów oraz skończyło się działanie kierunkowskazów
  else if ( Licznik_KierunkowskazyLewy == 0 && Licznik_KierunkowskazyPrawy == 0 ) {
    // Ustawiamy ilość cykli przerwy w działaniu kierunkowskazów
    Licznik_KierunkowskazyPrzerwa = Cykle_KierunkowskazyPrzerwa;
  }
  
  // Operacje do wykonania jeśli aktualnie jest przerwa świateł stop
  if ( Licznik_SwiatlaStopPrzerwa > 0 ) {
    // Zmniejszamy ilość cykli, które mają być jeszcze wykonane
    Licznik_SwiatlaStopPrzerwa = Licznik_SwiatlaStopPrzerwa - 1;
    // Jeśli przerwa się skończyła to uruchamiamy światła stop
    if ( Licznik_SwiatlaStopPrzerwa == 0 ) {
      digitalWrite(PIN_SwiatlaStop, HIGH); // Włączanie świateł stop
      // Ustawiamy licznik ile cykli będą działać światła stop
      Licznik_SwiatlaStop = Cykle_SwiatlaStop;
    }
  }
  // Operacje do wykonania jeśli nie było przerwy w działaniu świateł stop oraz jeśli aktualnie działają światła stop
  else if ( Licznik_SwiatlaStop > 0 ) {
    // Zmniejszamy ilość cykli, które mają być jeszcze wykonane
    Licznik_SwiatlaStop = Licznik_SwiatlaStop - 1;
    // Jeśli skończyło się świecenie to gasimy światła stop
    if ( Licznik_SwiatlaStop == 0) {
      digitalWrite(PIN_SwiatlaStop, LOW); // Wyłączanie świateł stop
      // Ustawiamy licznik ile cykli będzie przerwy świateł stop
      Licznik_SwiatlaStopPrzerwa = Cykle_SwiatlaStopPrzerwa;
    }
  }

  // Operacje do wykonania jeśli aktualnie jest przerwa świateł drogowych
  if ( Licznik_SwiatlaDrogowePrzerwa > 0 ) {
    // Zmniejszamy ilość cykli, które mają być jeszcze wykonane
    Licznik_SwiatlaDrogowePrzerwa = Licznik_SwiatlaDrogowePrzerwa - 1;
    // Jeśli przerwa się skończyła to uruchamiamy światła drogowe
    if ( Licznik_SwiatlaDrogowePrzerwa == 0 ) {
      digitalWrite(PIN_SwiatlaDrogowe, HIGH); // Włączanie świateł drogowych
      // Ustawiamy licznik ile cykli będą działać światła drogowe
      Licznik_SwiatlaDrogowe = Cykle_SwiatlaDrogowe;
    }
  }
  // Operacje do wykonania jeśli nie było przerwy w działaniu świateł drogowych oraz jeśli aktualnie działają światła drogowe
  else if ( Licznik_SwiatlaDrogowe > 0 ) {
    // Zmniejszamy ilość cykli, które mają być jeszcze wykonane
    Licznik_SwiatlaDrogowe = Licznik_SwiatlaDrogowe - 1;
    // Jeśli skończyło się świecenie to gasimy światła drogowe 
    if ( Licznik_SwiatlaDrogowe == 0 ) {
      digitalWrite(PIN_SwiatlaDrogowe, LOW); // Wyłączanie świateł drogowych
      // Ustawiamy licznik ile cykli będzie przerwy świateł drogowych
      Licznik_SwiatlaDrogowePrzerwa = Cykle_SwiatlaDrogowePrzerwa;
    }
  }

}
